<?php

namespace TVn7\ForumBundle\Controller;


use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Criteria;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\ParamConverter;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use TVn7\ForumBundle\Entity\Forum;
use TVn7\ForumBundle\Entity\Post;
use TVn7\ForumBundle\Entity\Reaction;
use TVn7\ForumBundle\Entity\Topic;
use TVn7\ForumBundle\Form\PostType;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;

/**
 * Class PostController
 * @package TVn7\ForumBundle\Controller
 * @Security("has_role('ROLE_FORUM_USER')")
 */
class PostController extends Controller
{

    /**
     * @param Forum $forum
     * @param Topic $topic
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\Response
     * @ParamConverter("forum", class="TVn7ForumBundle:Forum", options={"mapping" : {"forum" : "slug"}})
     */
    public function newAction(Forum $forum, Topic $topic, Request $request)
    {
        
        if ($topic->getForum() != $forum) {
            throw $this->createNotFoundException();
        }

        if ($forum->isDeleted() || $topic->isDeleted()) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted('reply', $topic);

        $post = new Post();
        $form = $this->createForm(PostType::class, $post);

        $user = $this->getUser();

        if ($request->getMethod() == 'POST') {
            $form->handleRequest($request);
            $post->setUser($user);
            $post->setTopic($topic);
            $topic->addSubscriber($user);
            if ($form->isValid()) {
                $em = $this->getDoctrine()->getManager();
                $em->persist($post);
                $em->persist($topic);
                $em->flush();

                $notifier = $this->get("t_vn7_forum.ForumNotifier");
                $notifier->notifyNewReply($topic, $post, $user);

                $this->addFlash('success', 'Le commentaire a bien été enregistré.');

                return $this->redirectToRoute("_tdb_forum_topic_show", array("forum" => $forum->getSlug(), "slug" => $topic->getSlug()));
            }
        }

        return $this->render('TVn7ForumBundle:Post:new.html.twig', array(
                'topic' => $topic,
                'form' => $form->createView()
            )
        );

    }

    /**
     * @param Forum $forum
     * @param Topic $topic
     * @param Post $post
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     *
     * @ParamConverter("forum", class="TVn7ForumBundle:Forum", options={"mapping" : {"forum" : "slug"}})
     * @ParamConverter("topic", class="TVn7ForumBundle:Topic", options={"mapping" : {"topic" : "slug"}})
     */
    public function editAction(Forum $forum, Topic $topic, Post $post, Request $request)
    {     
        if ($topic->getForum() != $forum || $post->getTopic() != $topic) {
            throw $this->createNotFoundException();
        }

        if ($forum->isDeleted() || $topic->isDeleted()) {
            throw $this->createNotFoundException();
        }

        $user = $this->getUser();
        $this->denyAccessUnlessGranted('edit',$post);
        
        $form = $this->createForm(PostType::class, $post);
        if ($request->getMethod() == 'POST') {
            $form->handleRequest($request);
            $post->setEditBy($user);
            if ($form->isValid()) {
                $em = $this->getDoctrine()->getManager();
                $em->persist($post);
                $em->flush();

                $this->addFlash('success', 'La réponse a bien été modifiée.');

                return $this->redirectToRoute("_tdb_forum_topic_show", array("forum" => $forum->getSlug(), "slug" => $topic->getSlug()));
            }
        }

        return $this->render('TVn7ForumBundle:Post:edit.html.twig', array(
            'form' => $form->createView(),
            'topic' => $topic
        ));
    }

    /**
     * @param Forum $forum
     * @param Topic $topic
     * @param Post $post
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     *
     * @ParamConverter("forum", class="TVn7ForumBundle:Forum", options={"mapping" : {"forum" : "slug"}})
     * @ParamConverter("topic", class="TVn7ForumBundle:Topic", options={"mapping" : {"topic" : "slug"}})
     */
    public function deleteAction(Forum $forum, Topic $topic, Post $post, Request $request)
    {
        
        
        if ($topic->getForum() != $forum || $post->getTopic() != $topic) {
            throw $this->createNotFoundException();
        }

        if ($forum->isDeleted() || $topic->isDeleted()) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted('delete',$post);

        $post->setDeleted(true);
        $em = $this->getDoctrine()->getManager();
        $em->persist($post);
        $em->flush();

        $this->addFlash('success', 'La réponse a bien été supprimée.');

        return $this->redirectToRoute("_tdb_forum_topic_show", array("forum" => $forum->getSlug(), "slug" => $topic->getSlug()));

    }

     /**
     * @param Forum $forum
     * @param Topic $topic
     * @param Post $post
     * @param $reactionName
     * @param Request $request
     * @return JsonResponse
     *
     * @ParamConverter("forum", class="TVn7ForumBundle:Forum", options={"mapping" : {"forum" : "slug"}})
     * @ParamConverter("topic", class="TVn7ForumBundle:Topic", options={"mapping" : {"topic" : "slug"}})
     */
    public function reactAction(Forum $forum, Topic $topic, Post $post, $reactionName,Request $request)
    {
        if ($topic->getForum() != $forum || $post->getTopic() != $topic) {
            throw $this->createNotFoundException();
        }

        if ($forum->isDeleted() || $topic->isDeleted()) {
            throw $this->createNotFoundException();
        }

        $em = $this->getDoctrine()->getManager();
        $isdelete = false;
        $prevreaction = $post->getReactions()->filter(function ($reaction){
            return $reaction->getUsers()->contains($this->getUser());
        });
        if ($prevreaction->count() > 0) {
            $prevreaction = $prevreaction->first();
            $isdelete = $prevreaction->getReactionName() == $reactionName;
            $prevreaction->getUsers()->removeElement($this->getUser());
            if($prevreaction->getUsers()->count() == 0) {
                $em->remove($prevreaction);
            }else{
                $em->persist($prevreaction);
            }

        }

        if(!$isdelete){
            $reaction = $post->getReactions()->filter(function ($reaction) use ($reactionName) {
                return $reaction->getReactionName() == $reactionName;
            });

            if ($reaction->count() == 0) {
                $reaction = new Reaction();
                $reaction->setReactionName($reactionName);
                $reaction->setPost($post);
                $post->getReactions()->add($reaction);
            } else {
                $reaction = $reaction->first();
            }
            $reaction->getUsers()->add($this->getUser());
            $em->persist($reaction);
        }
        $em->persist($post);
        $em->flush();
        $response = new JsonResponse();
        $data = new ArrayCollection();
        foreach($post->getReactions() as $reaction){
            $userlist = array();
            foreach($reaction->getUsers() as $user){
                array_push($userlist,$user->getNomComplet());
            }
            $data->set($reaction->getReactionName(), $userlist);
        }

        $response->setData($data->toArray());
        return $response;
    }

}